%% DESCRIPTION
% Calculate the difference of intensity histograms and find the
% 'non-linearity' of the histogram change, defined by the product of the
% sum of the bin differences.
% The idea is that a high change in the histograms is a good indicator for
% a scene shift.
% Furthermore we 'normalize' the histogram change by dividing by the
% average histogram difference of the last N frame shifts.
% Finally a combination is calculated, i.e. the product of the normalized
% 'non-linearity' and the 'non-linearity'.

close all;
if(~exist('small_mov','var'))
    clear all;
    run('../copy_movie_to_smaller_array.m');
end

nElements = smallHeight*smallWidth;

%% INTIIALIZE
if(size(small_mov(1).cdata,3) == 3)
    grayScale = cast(rgb2gray(small_mov(1).cdata),'double');
else
    grayScale = cast(small_mov(1).cdata,'double');
end
secondLastBinElements = hist(grayScale(:),20);
secondLastBinElements = secondLastBinElements / nElements;

if(size(small_mov(2).cdata,3) == 3)
    grayScale = cast(rgb2gray(small_mov(2).cdata),'double');
else
    grayScale = cast(small_mov(2).cdata,'double');
end
lastBinElements = hist(grayScale(:),20);
lastBinElements = lastBinElements / nElements;

%% CALCULATE HISTOGRAM NON-LINEARITY
nonlinearityHistory = zeros(nFrames-1,1);
averagingLength = 20;
averagedNonlinearityHistory = zeros(nFrames-averagingLength,1);
for i=3:nFrames
    if(size(small_mov(i).cdata,3) == 3)
        grayScale = cast(rgb2gray(small_mov(i).cdata),'double');
    else
        grayScale = cast(small_mov(i).cdata,'double');
    end
    currentBinElements = hist(grayScale(:),20);
    currentBinElements = currentBinElements / nElements;
    nonlinearity = sum(abs(lastBinElements - currentBinElements)) ...
                        *sum(abs(secondLastBinElements-currentBinElements));
    nonlinearityHistory(i-1) = nonlinearity;
    if(i >= averagingLength+1)
        averagedNonlinearityHistory(i-averagingLength) = nonlinearity ...
                    / mean(nonlinearityHistory((i-averagingLength):(i-1)));
    end
%     disp(strcat('Frame ',num2str(i),' nonlinearity: ',num2str(nonlinearity)));
%     loglog(lastBinElements,currentBinElements,'O');
%     waitforbuttonpress;
    secondLastBinElements = lastBinElements;
    lastBinElements = currentBinElements;
end
combinedAvgNonlinearityHistory = averagedNonlinearityHistory.*nonlinearityHistory(averagingLength:(nFrames-1));

plot(2:nFrames,nonlinearityHistory);
title('Intensity histogram change');
xlabel('# Frame');
ylabel('Change');
waitforbuttonpress;
hold on;
plot((averagingLength+1):nFrames,averagedNonlinearityHistory,'red');
title('Averaged intensity histogram change');
waitforbuttonpress;
hold off;
plot((averagingLength+1):nFrames,combinedAvgNonlinearityHistory);
title('Combined intensity histogram change');
xlabel('# Frame');
ylabel('Change');