%% DESCRIPTION
% This script shows the mean camera or object movement based on the
% displacement of the SIFT features. A combination of two adjacent frames
% and the matched SIFT features is shown, as well as an historgram of the
% x- and y-displacement distribution and a plot with a 10 frame-history
% (trace) of the average displacement.

close all;
if(~exist('small_mov','var'))
    clear all;
    run('../copy_movie_to_smaller_array.m');
end
addpath('..');

cutThreshold = 0.0019;


%% Perform cut detection
% For Bang! You're dead
trueCuts = [ 188, 208, 225, 261, 296, 333, 374, 387, 410, 427, 442, 479, 500, 567, 589, 613, 628, 649, 662, 673, 694, 698, 739, 756, 771, 805, 819, 832, 841, 854, 945, 1159, 1206, 1503, 1607, 2122, 2208, 2356, 2435, 2523, 2567, 2593, 2607, 2628, 2658, 2692, 2724, 2750, 2784, 2805, 2937 ];
tic;
[ cutsDetected, cuts, matchCountMatrix, diffMatchCountMatrix, diffThreshold ] = sift_cutdetection(small_mov,1,cutThreshold);
toc;
truePositives = sum(ismember(cuts,trueCuts));
precision = truePositives / length(cuts)
recall = truePositives / length(trueCuts)
[ cutsAdaptive, cutsDetectedAdaptive ] = cut_detection_frame_distance(diffMatchCountMatrix,0.4);
truePositives = sum(ismember(cutsAdaptive,trueCuts));
precision = truePositives / length(cutsAdaptive)
recall = truePositives / length(trueCuts)

t = 1:(nFrames-1);
reportPlot('plot','SIFT matches between adjacent frames',t,'Frame',matchCountMatrix,'# Matches');
t = 2:(nFrames-1);
reportPlot('plot','Difference of SIFT matches between adjacent frames',t,'Frame',[ diffMatchCountMatrix diffThreshold ],'Difference');

% figure();
% for cut=1:cutsDetected
%     frameIdx = cuts(cut);
%     imagesc(cat(2,small_mov(frameIdx-1).cdata,small_mov(frameIdx).cdata));
%     title(strcat('Frames ',num2str(frameIdx-1),' + ',num2str(frameIdx)));
%     waitforbuttonpress;
% end
return;

%% Determine best thresholds
% For Zelotypia
trueCuts = [ 16, 67, 137, 187, 244, 272, 347, 399, 416 ];
F1score = zeros(20,1);
[ ~, ~, ~, diffMatchCountMatrix, ~ ] = sift_cutdetection(small_mov,1,cutThreshold);
for i=1:20
    cutThreshold = 0.05*i;
    [ cuts, cutsDetected ] = cut_detection_frame_distance(diffMatchCountMatrix,cutThreshold);
    truePositives = sum(ismember(cuts,trueCuts));
    precision = truePositives / length(cuts);
    recall = truePositives / length(trueCuts);
    F1score(i) = 2*precision*recall / (precision + recall);
end
figure, plot(0.05:0.05:1,F1score,'r*-'); title('Adaptive threshold');