%% DESCRIPTION
% Generate artificial data and find alternative leverage scores based on
% the local observation density. Show that these alternative leverage scores
% can be used for subsampling.

close all;
clear all;
run('../plot_settings.m');
addpath('..');

n = 1000;
p = 2;
noiseLevel = 9;

% [X, y, ~ ] = generate_data(n,p,noiseLevel,'t',1);
[X, y, ~ ] = generate_data(n,p,noiseLevel,'normal');
beta = ((X'*X) \ X') * y;

K = 10;
medianKneighborDistance = zeros(n,1);
meanKneighborDistance = zeros(n,1);
% Find the distance between all pairs of observations
for i=1:n
    distance = sqrt(sum((X-repmat(X(i,:),n,1)).^2,2));
    idxDistanceMatrix = [ (1:n)' , distance ];
    sortedIdxDistanceMatrix = sortrows(idxDistanceMatrix,2);
    % Find median and mean. We use 2:11 to discard the distance to the
    % observation itself, which is always 0.
    medianKneighborDistance(i) = median(sortedIdxDistanceMatrix(2:(1+K),2));
    meanKneighborDistance(i) = mean(sortedIdxDistanceMatrix(2:(1+K),2));
end
% We can either use the median or the mean distance to the K-nearest
% observations:
% h = medianKneighborDistance / sum(medianKneighborDistance);
h = meanKneighborDistance / sum(meanKneighborDistance);
leverage_visualization(X,h);

H = X*((X'*X) \ X');
leverageScores = diag(H);

reportPlot('scatter','KNN importance measure vs. leverage scores',...
            leverageScores,'Leverage scores',...
            h,'KNN importance measure');
return;

%% Simulate subsampling for uniform and importance measure sampling
showScatterPlots = 0;
experiments = 20;
repetitions = 100;
conditional = 1;
stepSize = 50;
linear_mean_var_experiments_uniform;
linear_mean_var_experiments_leverage;