%% DESCRIPTION
% Find the computation time for different types of linear regressions and
% sampling

clear all;
close all;
run('../plot_settings.m');

% Set parameters
n = 10000;
p = 100;
noiseLevel = 9; % Noise level of the respnse
distribution = 'normal'; % Can either be 'normal' or 't'
degreeOfFreedom = 1; % Has only effect if distribution == 't'
repetitions = 100; % Repeat this many times for each sample size and method.
experiments = 10; % Number of sample sizes

timeVector = zeros(experiments,3);
for k=1:experiments
    % Set sample size
    n = 1000*k;
    % Generate data
    [ X, Y, beta ] = generate_data(n,p,noiseLevel,distribution,degreeOfFreedom);

    %% FULL LINEAR REGRESSION
    disp('Full linear regression');
    tic;
    for i=1:repetitions
        betaHat = (X'*X) \ (X'*Y);
    end
    timeVector(k,1) = toc;

    %% UNIFORM HALF SAMPLE REGRESSION
    disp('Uniform half sample linear regression');
    tic;
    for i=1:repetitions
        sample = rand(n/2,1) < 0.5;
        Xsample = X(sample,:);
        betaHat = (Xsample'*Xsample) \ (Xsample'*Y(sample));
    end
    timeVector(k,2) = toc;

    %% LEVERAGE HALF SAMPLE REGRESSION
    disp('Leverage half sample linear regression');
    tic;
    for i=1:repetitions
        H = X*((X'*X) \ X');
        sample = nonuniform_sampling(diag(H) / p,n/2);
        Xsample = X(sample,:);
        betaHat = (Xsample'*Xsample) \ (Xsample'*Y(sample));
    end
    timeVector(k,3) = toc;
end
figure, semilogy(1000*(1:experiments),timeVector/repetitions,'-','LineWidth',2), ...
    xlabel('Observations'),...
    ylabel('Time (s)'),...
    legend('Full regression','Uniform sampling','Leverage sampling','Location','NorthWest'),...
    grid on;