function [ interval ] = confidenceInterval( data )
%CONFIDENCEINTERVAL Find the confidence interval for a the columns in a dataset
%   Method 1:   95% bounds of the data
%   Method 2:   Bootstrap estimate of the standard deviation

method = 2;

if(method == 1)
    L = size(data,1);
    meanValue = mean(data,1);
    sortedData = sort(data);
    startIndex = max(1,floor(0.025*L));
    stopIndex = ceil(0.975*L);

    columns = size(data,2);
    interval = zeros(columns,2);
    for i=1:columns
        interval(i,1) = meanValue(i)-sortedData(startIndex,i);
        interval(i,2) = sortedData(stopIndex,i)-meanValue(i);
    end
elseif(method == 2)
    L = size(data,1);
    
    columns = size(data,2);
    interval = zeros(columns,2);
    for i=1:columns
        M = 100; % Bootstrap experiments
        bootstrapMeans = zeros(M,1);
        for j=1:M
            sample = ceil(L*rand(M,1));
            bootstrapData = data(i,sample);
            bootstrapMeans(j) = mean(bootstrapData);
        end
        meanBootstrapMean = mean(bootstrapMeans);
        stdBootstrapMeans = std(bootstrapMeans);
        
        interval(i,1) = meanBootstrapMean - stdBootstrapMeans;
        interval(i,2) = meanBootstrapMean + stdBootstrapMeans;
    end
else
    columns = size(data,2);
    interval = zeros(columns,2);
end

end

