%% DESCRIPTION
% Calculate the generalized leverage scores for logistic regression, using
% the objective function and example-data (see thesis for details).

close all;
clear all;

run('../plot_settings.m');

n = 1000;
p = 2;


%% X-data definitions:
% #### Definition 1: #####
%offset = 5*rand(1,p);
%X = [ randn([ n/2, p ]) +  kron(ones(n/2,1),offset) ; 2*randn([ n/2, p ]) ];

% #### Definition 2: #####
%X = randn([ n, p ]);


MU = ones(n,p);
SIGMA = zeros(p,p);
for i=1:p
    for j=1:p
        SIGMA(i,j) = 2^(-abs(i-j)+1);
    end
end
% #### Definition GA: #####
X = mvnrnd(MU,SIGMA);

% #### Definition T3: #####
%X = mvtrnd(SIGMA,3,n);

% #### Definition T1: #####
%X = mvtrnd(SIGMA,1,n);

% Centering:
for i=1:p
    X(:,i) = X(:,i) - mean(X(:,i));
end
Xmax = max(max(X(:,1)))*1.1;
Xmin = min(min(X(:,1)))*1.1;
Ymax = max(max(X(:,2)))*1.1;
Ymin = min(min(X(:,2)))*1.1;

% data-distance from center:
Xdist = sum(X.^2,2);
distFig = figure();
hist(Xdist,50);

% Beta & noise
beta = [ 0; ones(p,1) ];
beta = beta / sum(beta.^2);
noise = normrnd(0,1,[ n, 1 ]);

%% Hat matrix:
W = zeros(n,n);
for i=1:n
    exponent = [ 1 X(i,:) ] * beta;
    W(i,i) = (1/(1 + exp(exponent))) * (1 - 1/(1 + exp(exponent)));
end

H = sqrt(W) * X * ((X'*W*X) \ X' * sqrt(W));
h = diag(H);
h = h / sum(h);

hatHistFig = figure();
hist(h,0.00025:0.0005:0.00975);


%% Response vector with noise:
y = ([ ones(n,1) X ] * beta + noise) > 0;

if( p == 2 )
    dataFig = figure();
    scatter(X(:,1),X(:,2),20,y);
    axis([Xmin Xmax Ymin Ymax]);
    
    levScoreFig = figure();
    scatter(X(:,1),X(:,2),20,h);
    axis([Xmin Xmax Ymin Ymax]);
end

%% Generalized leverage scores (analytical approach, see thesis):
% Q: The objective function
% a: The regression parameter (here equal to \beta).

X2 = [ ones(n,1) X ];

% Qda = zeros(p+1,1);
% 
% for i=1:(p+1)
%     sumV = 0;
%     for j=1:n
%         sumV = sumV + (y(j) - X(j,:)*beta)*X(j,i);
%     end
%     Qda(i) = -2*sumV;
% end

Qdada = zeros(p+1,p+1);

for i=1:(p+1)
    for j=1:(p+1)
        sumV = 0;
        for k=1:n
            sumV = sumV + X2(k,i)*X2(k,j)*exp(X2(k,:)*beta) / (1 + exp(X2(k,:)*beta))^2;
        end
        Qdada(i,j) = sumV;
    end
end

QdadY = zeros(p+1,n);

for i=1:(p+1)
    for j=1:n
        QdadY(i,j) = X2(j,i);
    end
end

Da = zeros(n,p+1);

for i=1:n
    for j=1:(p+1)
        Da(i,j) = X2(i,j)*beta(j);
    end
end

GL = Da * (-Qdada \ QdadY);
GLdiag = abs(diag(GL));
GLdiag = GLdiag / sum(GLdiag);

GLhistFig = figure();
hist(GLdiag,0.00025:0.0005:0.00975);

if(p==2)
    figure();
    scatter(X(:,1),X(:,2),plotScatterSize,GLdiag);
    axis([Xmin Xmax Ymin Ymax]);
    caxis([ 0 0.01 ]);
end

hatGLcompareFig = figure();
plot(h,GLdiag,'O');

% Show leverage scores
tri = delaunay(X(:,1),X(:,2));
figure();
trisurf(tri,X(:,1),X(:,2),GLdiag);
figure();
trisurf(tri,X(:,1),X(:,2),h);