%% DESCRIPTION
% This script uses stocastic simulation to find the generalized leverage
% scores for linear regression and a given data set. The method is to run 
% several sampled linear regression models for each observation and
% determining if the prediction of this observation is changed if its label
% is changed.

close all;
clear all;
addpath('..');
run('../plot_settings.m');

% Set data parameters
n = 1000;
p = 2;
noiseLevel = 9;

% Generate data and leverage scores
[ X, Y ] = generate_data(n,p,noiseLevel,'normal');
H = X*((X'*X) \ X');
h = diag(H);

% Set algorithm parameters (see thesis for details)
M = 200;        % Subset size
k = 100;         % Simulations per observation


derivative = zeros(n,k);
delta = 1;
for j=1:n
    for c1=1:k
        % Draw subset of size M
        sample = ceil(n*rand(M,1));
        Xsample = [X(sample,:) ; X(j,:) ];
        Ysample = [Y(sample) ; Y(j) ];
        
        % Train model
        beta = (Xsample'*Xsample) \ (Xsample'*Ysample);
        value1 = Xsample(M+1,:) * beta;
        
        % Modify the response vector Y
        Ysample(M+1) = Ysample(M+1) + delta;
        
        % Re-train the model
        beta = (Xsample'*Xsample) \ (Xsample'*Ysample);
        value2 = Xsample(M+1,:) * beta;
        
        % Store the derivative
        derivative(j,c1) = (value2 - value1)/delta;
        
    end
end

derivativeMean = mean(derivative,2);
derivativeVar = var(derivative,0,2);

% Visualize the results
%figure, scatter(X(:,1),X(:,2),20,derivativeMean);
leverage_visualization(X,derivativeMean);
reportPlot('scatter','Comparison of stocastic and hat matrix leverage',...
    derivativeMean,'Stocastic leverage',...
    h,'Hat matrix leverage');

