%% DESCRIPTION
% Perform subsampled linear regression and determine the bias and variance
% of the regression parameter \beta. The samples are drawn accoring to
% different weightings: Uniform, reweighted leverage scores, unweighted
% leverage scores and one alternative K-nearest neighbors weight (see
% thesis for details)

close all;
clear all;
run('../plot_settings.m');
addpath('..');

% Set data parameters
n = 1000;
p = 10;
distribution = 't';
df = 3;
noiseLevel = 9;

% Set experiment parameters
experiments = 20; % Number of sample sizes to test
stepSize = 50; % Step size between sample sizes. Example 50: 50, 100, 150,...
repetitions = 1000; % Repeat each experiment several times and average the bias and variance results
bootstrapRepetitions = 100; % Size of the bootstrap for estimating the standard
                            % deviation of thebias- and variance estimates
conditional = 0; % 0: Unconditional regression, 1: Conditional regression (fixed data set and response)

showScatterPlots = 0; % If p==2, do you want to see the scatter plots for each experiment?

%% Data generation
[ X, y, beta0 ] = generate_data(n,p,noiseLevel,distribution,df);
noise = normrnd(0,noiseLevel,[ n, 1 ]);

%% Hat matrix leverage scores:
H = X * ((X'*X) \ X');
h = diag(H);
h = h / sum(h);

% greatestLeverage = [ (1:n)', h ];
% sortedGreatestLeverage = sortrows(greatestLeverage,2);
% noise(sortedGreatestLeverage(1:10,1)) = 0;

% Xdist = sqrt(sum(X.^2,2));
% reportPlot('scatter','Leverage scores versus distance to mean',...
%             Xdist,'Distance',...
%             sqrt(h),'Square root of leverage scores');
% return;

% hatHistFig = figure();
% hist(h,20);
% return;
 
% if(p==2)
%     figure();
%     scatter(X(:,1),X(:,2),plotScatterSize,h);
%     axis([Xmin Xmax Ymin Ymax]);
% end

%% Response vector with noise (deprecated, now handled by generate_data):
%y = X * beta + (meanH./h).*noise; % Only leverage-dependent noise can influence
                                  % difference between uniform and leverage-based sampling
                                  % for multivariate distribution.

%  y = X * beta0 + noise;       % This gives the desired graphs for the multivariate-
                            % t-distributions with low degree of freedom.
                            
                            
%% Produce results
linear_mean_var_experiments_uniform;
linear_mean_var_experiments_leverage_rescaled;
linear_mean_var_experiments_leverage;
linear_mean_var_experiments_knnImportanceMeasure;