%% DESCRIPTION
% Perform subsampled logistic regression and determine the bias and variance
% of the regression parameter \beta. The samples are drawn accoring to
% different weightings: Uniform and unweighted leverage scores. Unweighted
% leverage scores can also be replaced by the W-matrix.

close all;
clear all;
run('../plot_settings.m');
addpath('..');

% Set data parameters
n = 500;
p = 2;

% Set experiment parameters
experiments = 20; % Number of sample sizes to test
stepSize = 50; % Step size between sample sizes. Example 50: 50, 100, 150,...
sampleSizeVector = [ 10, 25, stepSize:stepSize:stepSize*(experiments-2) ]; % Define the sample sizes to be tested
repetitions = 50; % Repeat each experiment several times and average the bias and variance results
bootstrapRepetitions = 100; % Size of the bootstrap for estimating the standard
                            % deviation of thebias- and variance estimates
conditional = 0; % 0: Unconditional regression, 1: Conditional regression (fixed data set and response)

showScatterPlots = 0; % If p==2, do you want to see the scatter plots for each experiment?

%% X-data definitions:
% #### Definition 1: #####
%offset = 5*rand(1,p);
%X = [ randn([ n/2, p ]) +  kron(ones(n/2,1),offset) ; 2*randn([ n/2, p ]) ];

% #### Definition 2: #####
%X = randn([ n, p ]);

% #### Definition GPA: #####
MU = ones(n,p);
SIGMA = zeros(p,p);
for i=1:p
    for j=1:p
        SIGMA(i,j) = 2^(-abs(i-j)+1);
    end
end
X = mvnrnd(MU,SIGMA);

% #### Definition T1: #####
% X = mvtrnd(SIGMA,3,n);

% #### Definition T1: #####
% X = mvtrnd(SIGMA,1,n);

% Center:
% for i=1:p
%     X(:,i) = X(:,i) - mean(X(:,i));
% end
Xmax = max(max(X(:,1)))*1.1;
Xmin = min(min(X(:,1)))*1.1;
Ymax = max(max(X(:,2)))*1.1;
Ymin = min(min(X(:,2)))*1.1;

% Beta & noise
beta0 = [ 0; ones(p,1) ];
% noise = normrnd(0,1,[ n, 1 ]); % Sigma chosen to create a decent overlap.

%% W-matrix: and hat matrix
W = zeros(n,n);
for i=1:n
    exponent = [ 1 X(i,:) ] * beta0;
    W(i,i) = (1/(1 + exp(exponent))) * (1 - 1/(1 + exp(exponent)));
end
h = diag(W) / sum(diag(W)); 

% Calculate the hat matrix (leave commented to use W-matrix as weighting)
% H = sqrt(W) * X * ((X'*W*X) \ X' * sqrt(W));
% h = diag(H);
% h = h / sum(h);

% Visualize leverage scores
hatHistFig = figure();
hist(h,20);

%% Response vector
% Calculate ideal response and class probabilities
y0 = [ ones(n,1) X ] * beta0;
p0 = exp(y0) ./ (1+exp(y0));
p0(isnan(p0)) = 1;

% Calculate new response from class probabilities:
y = (rand(n,1) > p0) + 1;
betaOLS = mnrfit(X,y);

% Visualize data if p==2:
if( p == 2)
    dataFig = figure();
    scatter(X(:,1),X(:,2),20,y);
    axis([Xmin Xmax Ymin Ymax]);
    
    levScoreFig = figure();
    scatter(X(:,1),X(:,2),20,h);
    axis([Xmin Xmax Ymin Ymax]);
end

%% Produce results
logistic_mean_var_experiments_uniform;
logistic_mean_var_experiments_leverage;