%% DESCRIPTION
% Find the slope and offset of the SVM separation line for different sample
% sizes. Find the bias and variance for these two parameters.
% The data is 2D for visualization purposes.

close all;
clear all;
addpath('..');

% Load precalculated leverage scores and data:
load('../../dataset/generalizedLeverage_SVM_n1000_p2_m200_k100.mat');
stocasticLeverage = abs(derivativeMean) / sum(abs(derivativeMean));
leverageScores = 0.9*stocasticLeverage + 0.1/1000;
leverageScores = leverageScores / sum(leverageScores);

% Train the full SVM model
fullSVMModel = svmtrain(X,Y);
fullSlopeNormal = fullSVMModel.Alpha'*fullSVMModel.SupportVectors;
fullSlope = -fullSlopeNormal(1) / fullSlopeNormal(2);
fullIntercept = fullSVMModel.Bias;

%=================================
% Subsample-experiments
repetitions = 50;
sampleSizeSteps = 20;
sampleSizeStepSize = 50;
betaHistory = zeros(sampleSizeSteps,repetitions,2);

%% Perform experiments
for sampleSizeIdx=1:sampleSizeSteps % Loop over sample sizes
    M = sampleSizeStepSize*sampleSizeIdx;
    disp(M);
    for experiment=1:repetitions % Perform the experiments for each sample size several times
        sample = nonuniform_sampling(leverageScores,M);
        % Build subsampled data set
        Xsample = X(sample,:);
        Ysample = Y(sample);
        
        % Find slope and offset (intercept) for the subsampled SVM problem
        sampleSVMModel = svmtrain(Xsample,Ysample);
        sampleSlopeNormal = sampleSVMModel.Alpha'*sampleSVMModel.SupportVectors;
        sampleSlope = -sampleSlopeNormal(1) / sampleSlopeNormal(2);
        sampleIntercept = sampleSVMModel.Bias;
        betaHistory(sampleSizeIdx,experiment,1) = sampleSlope;
        betaHistory(sampleSizeIdx,experiment,2) = sampleIntercept;
    end
end
% Find bias and variance...
% ...for the slope
slopeBiasVector = mean(betaHistory(:,:,1),2)-fullSlope;
slopeVarVector = var(betaHistory(:,:,1),0,2);
% ...for the intercept
interceptBiasVector = mean(betaHistory(:,:,2),2)-fullIntercept;
interceptVarVector = var(betaHistory(:,:,2),0,2);

%% Visualize results
% For the slope:
reportPlot('plot','Separation line slope bias',...
            sampleSizeStepSize:sampleSizeStepSize:sampleSizeStepSize*sampleSizeSteps,'Sample size',...
            slopeBiasVector,'Bias');
hold on; plot([0 1000],[ 0 0],'r--','LineWidth',2);hold off;
reportPlot('plot','Separation line - Logarithmic slope variance vs sample size',...
            sampleSizeStepSize:sampleSizeStepSize:sampleSizeStepSize*sampleSizeSteps,'Sample size',...
            log(slopeVarVector),'log(slope variance)');