%% DESCRIPTION
% Visualize the logistic regression likelihood function for 1-d data.

close all;
clear all;

X = (1:50)';
y = [ ones(25,1); 2*ones(25,1) ];

betaHat = mnrfit(X,y);
yhat = ([ ones(50,1) X ] * betaHat) < 0;

figure();
plot(X,y);
figure();
plot(X,yhat);

xgv = 80:2:180;
ygv = -6:0.1:-3;
width = length(xgv);
height = length(ygv);
[ Xgrid, Ygrid ] = meshgrid(xgv,ygv);

likelihood = ones(height,width);
maxLikelihood = 0;
xMax = 1;
yMax = 1;
for i=1:width
    for j=1:height
        for n=1:length(X)
            prob = (1/(1+exp(Xgrid(j,i)+Ygrid(j,i)*X(n))));
            likelihood(j,i) = likelihood(j,i)*prob^(y(n)-1)*(1-prob)^(2-y(n));
        end
        if(likelihood(j,i) > maxLikelihood)
            maxLikelihood = likelihood(j,i);
            xMax = Xgrid(j,i);
            yMax = Ygrid(j,i);
        end
    end
end

maxI = [ xMax yMax ]

figure();
tri = delaunay(Xgrid,Ygrid);
trisurf(tri,Xgrid,Ygrid,likelihood);