%% DESCRIPTION
% This script extracts a random patch of fixed size from a frame and uses
% SIFT-features to detect its presence in another frame.

close all;
if(~exist('small_mov','var'))
    clear all;
    run('../copy_movie_to_smaller_array.m');
end
run('/home/jan/dtu/speciale/matlab/vlfeat-0.9.18/toolbox/vl_setup');
addpath('../leverageScores');

patchWidth = floor(0.3*smallWidth);
patchHeight = floor(0.3*smallHeight);
load('../cutdetection/bang-youre-dead_cuts.mat'); % Load cuts of the movie
cuts = [ 1, cuts, nFrames ];

nPatches = 1;
scene1 = 3;
scene2 = 5;
while(true)
    patchPositionX = floor((smallWidth-patchWidth)*rand(nPatches,1))+1;
    patchPositionY = floor((smallHeight-patchHeight)*rand(nPatches,1))+1;
    % Patch:
    for patch=1:nPatches
        % Select frame
        sceneLength = cuts(scene1+1)-cuts(scene1);
        frame = cuts(scene1) + floor(sceneLength*rand);
        Ia = single(small_mov(frame).cdata);

        patches = Ia(patchPositionY(patch):(patchPositionY(patch)+patchHeight-1),...
            patchPositionX(patch):(patchPositionX(patch)+patchWidth-1));
    end
    % Frame
    sceneLength = cuts(scene2+1)-cuts(scene2);
    frame = cuts(scene2) + floor(sceneLength*rand);
    Ib = single(small_mov(frame).cdata);

    % SIFT-features
    [ fa, da ] = vl_sift(patches);
    [ fb, db ] = vl_sift(Ib);
    [ matches, scores ] = vl_ubcmatch(da,db);
    displacement = zeros(size(matches,2),2);
    for m=1:size(matches,2)
        displacement(m,1) = -fa(1,matches(1,m)) + fb(1,matches(2,m));
        displacement(m,2) = -fa(2,matches(1,m)) + fb(2,matches(2,m));
    end

    figure(1);
    subplot(2,2,1);
    imagesc(patches), colormap(gray), axis image off;
    vl_plotframe(fa(:,matches(1,:)));
    title('Patch');
    subplot(2,2,3);
    scatter(displacement(:,1),displacement(:,2));
    title('Displacement of SIFT-features');
    subplot(2,2,[ 2, 4]);
    imagesc(Ib), colormap(gray),axis image off;
    hold on;
    vl_plotframe(fb(:,matches(2,:)));
    if(~isempty(displacement))
        rectangle('Position',...
           [median(displacement(:,1)), median(displacement(:,2)),...
           patchWidth, patchHeight],'EdgeColor','b','LineWidth',2);
    end
    hold off;
    title('Frame');
    waitforbuttonpress;

end

