%% DESCRIPTION
% This script shows the mean camera or object movement based on the
% displacement of the SIFT features. A combination of two adjacent frames
% and the matched SIFT features is shown, as well as an historgram of the
% x- and y-displacement distribution and a plot with a 10 frame-history
% (trace) of the average displacement.
% We also perform image stabilization based on the displacement

close all;
if(~exist('small_mov','var'))
    clear all;
    run('../copy_movie_to_smaller_array.m');
end
run('/home/jan/dtu/speciale/matlab/vlfeat-0.9.18/toolbox/vl_setup');
vl_setup demo
addpath('..');

visualize = 1;

motionDirection = zeros(10,2);
displacement = zeros(nFrames,2);
displacement(1,:) = [ 0, 0 ];
matchCountMatrix = zeros(nFrames-1,1);
diffMatchCountMatrix = zeros(nFrames-2,1);
diffThreshold = zeros(nFrames-2,1);
cuts = [];
cutsDetected = 0;
panning = [ 0 ];

if(visualize)
    figure(1), set(gcf,'Units','normalized','Position',[ 0.07, 0.70, 0.3 0.3 ]);
    figure(2), set(gcf,'Units','normalized','Position',[ 0.68, 0.7, 0.3 0.3 ]);
    figure(3), set(gcf,'Units','normalized','Position',[ 0, 0, 1 1 ]);
end


for i=1:(nFrames-1)
    Ia = single(small_mov(i).cdata);
    Ib = single(small_mov(i+1).cdata);
    [fa,da] = vl_sift(imresize(Ia,0.5));
    [fb,db] = vl_sift(imresize(Ib,0.5));
   
   [matches, scores] = vl_ubcmatch(da,db);
   matchCountMatrix(i) = size(matches,2);
   
   %% Cut detection
   if(i > 1)
       diffMatchCountMatrix(i-1) = abs(matchCountMatrix(i-1) - matchCountMatrix(i));
   end
   if(cutsDetected > 0 && i > 10 && i-cuts(cutsDetected) > 10)
       meanDiff = mean(diffMatchCountMatrix((i-10):(i-2)));
       varDiff = var(diffMatchCountMatrix((i-10):(i-2)));
       diffThreshold(i-1) = 200;%max(200,2*meanDiff + 4*sqrt(varDiff));
   elseif(i > 1)
       diffThreshold(i-1) = 200;
   end
  if(i > 1 && diffMatchCountMatrix(i-1) > diffThreshold(i-1))
      if(cutsDetected == 0 || ...
         (cutsDetected > 0 && i-cuts(cutsDetected) > 1))
           cutsDetected = cutsDetected + 1;
           cuts(cutsDetected) = i;
           panning(cutsDetected+1) = 0;
      end
  end
   
  
  %% Object tracking and displacement
    [drop, perm] = sort(scores, 'descend') ;
    matches = matches(:, perm) ;
    scores  = scores(perm) ;
    
    motionDirection(1:9,:) = motionDirection(2:10,:);
    motionDirection(10,:) = median(fa(1:2,matches(1,:))'-fb(1:2,matches(2,:))');
    displacement(i+1,:) = displacement(i,:) + motionDirection(10,:);
    if(displacement(i+1,1) > smallWidth*0.6 ...
        || displacement(i+1,2) > smallHeight*0.6)
        panning(cutsDetected+1) = 1;
    end
    if((cutsDetected > 0 && cuts(cutsDetected) == i) ...panning
        || displacement(i+1,1) > smallWidth*0.6 ...
        || displacement(i+1,2) > smallHeight*0.6)
        displacement(i+1,:) = [ 0, 0 ];
    end
    

    
    %% Visualization
    if(visualize)
        figure(1); clf;
        hist(fa(1:2,matches(1,:))'-fb(1:2,matches(2,:))',-50:5:50);
        figure(2); clf;
        plot(motionDirection(:,1),motionDirection(:,2),'O-',0,0,'r*');
        axis([-70 70 -70 70 ]);


        figure(3) ; clf ;
        imagesc(cat(2, Ia, Ib)) ;

        xa = fa(1,matches(1,:)) ;
        xb = fb(1,matches(2,:)) + size(Ia,2) ;
        ya = fa(2,matches(1,:)) ;
        yb = fb(2,matches(2,:)) ;

        hold on ;
        h = line([xa ; xb], [ya ; yb]) ;
        set(h,'linewidth', 1, 'color', 'b') ;

        vl_plotframe(fa(:,matches(1,:))) ;
        fb(1,:) = fb(1,:) + size(Ia,2) ;
        vl_plotframe(fb(:,matches(2,:))) ;
        axis image off, colormap gray;
        % Matching visualization:
        vl_demo_print('sift_match_2', 1) ;

        % Stabilized image:
        figure(4); clf ;
        imagesc(Ia), colormap gray, axis([ -displacement(i,1) 512-displacement(i,1) -displacement(i,2) 206-displacement(i,2) ]);
        waitforbuttonpress;
    end
    
end

t = 1:(nFrames-1);
reportPlot('plot','SIFT matches between adjacent frames',t,'Frame',matchCountMatrix,'# Matches');
t = 2:(nFrames-1);
reportPlot('plot','Difference of SIFT matches between adjacent frames',t,'Frame',[ diffMatchCountMatrix diffThreshold ],'Difference');