function [ cuts, cutsDetected ] = cut_detection_frame_distance( frameDistance )
%CUT_DETECTION_FRAME_DISTANCE Detect cuts from the distance between frames.
%   Adjacent frames from different frames are assumed to have a larger
%   distance (for a 30x40 version of the frame, each pixel treated as a
%   dimension).
%   We find the mean distance for a window of size 2*averagingLength+1
%   centered at the current frame. We then calculate the difference between
%   the mean when including the current frame and when excluding it.
%   We then find the maximum difference of the means. A cut is detected when
%   the difference between frames is above 33% of the maximum distance.

nFrames = length(frameDistance)+1;

cuts = [];
cutsDetected = 0;
averagingLength = 3;
diffMeanVector = zeros(nFrames-1,1);
for i=2:nFrames
    idxAll = max(1,i-averagingLength):min(nFrames-1,i+averagingLength);
    idxWithout = idxAll(find(idxAll ~= i));
    meanAll = mean(frameDistance(idxAll));
    meanWithout =  mean(frameDistance(idxWithout));
    diffMean = meanAll - meanWithout;
    diffMeanVector(i-1) = diffMean;
end
% Calculate the maximum distance
maxMeanDiff = max(diffMeanVector);
for i=2:nFrames
    if(diffMeanVector(i-1) > maxMeanDiff/3)
        cutsDetected = cutsDetected + 1;
        cuts(cutsDetected) = i+1;
    end
end
%figure,plot(diffMeanVector);

end

