%% DESCRIPTION
% This script generates data and demonstrates the distance distortion
% bounds using random projections (described in the Johnson-Lindenstrauss
% lemma)

clear all;
close all;
addpath('../leverageScores');
addpath('..');

% Set parameters
n = 1000;
p = 10000;
epsilon = 0.5;
loadPrecalculatedResults = 1;

% Generate data
if(~loadPrecalculatedResults)
    [X,~,~] = generate_data(n,p,1,'normal');
end

% Theoretical bounds for n=1000, in steps of 100.
theoreticalBounds = zeros(100,1);
for i=1:100
   eps = epsilon;
   kTarget = i*100;
   kComp = 9*log(n)/(eps^2-eps^3);
   if(kComp > kTarget)
       theoreticalBounds(i) = NaN;
       continue;
   end
   while(kComp < kTarget && eps > 0)
       eps = eps - 0.001;
       kComp = 9*log(n)/(eps^2-eps^3);
   end
   theoreticalBounds(i) = eps;
end

%% Calculate distance between frames
% distanceMatrix = zeros(n);
% for observation1=1:n
%     for observation2=(observation1+1):n
%         distanceMatrix(observation1,observation2) = sqrt(sum((X(observation1,:)-X(observation2,:)).^2));
%         distanceMatrix(observation2,observation1) = distanceMatrix(observation1,observation2);
%     end
% end

%% Random projection
if(loadPrecalculatedResults)
    load('min_max_std_rp_distortion');
else
    % Generate random matrix with normal distributed entries
    minVector = zeros(100,1);
    maxVector = zeros(100,1);
    upperStdVector = zeros(100,1);
    lowerStdVector = zeros(100,1);
    for dim=1:80
        disp(dim);
        %subDimension = ceil(9*log(n)/(epsilon^2-epsilon^3));
        subDimension = 100*dim;
        RP = mvnrnd(zeros(subDimension,p),eye(p));
        rpX = (1/sqrt(subDimension))*X*RP';
        rpDistanceMatrix = zeros(n);
        for observation1=1:n
            for observation2=(observation1+1):n
                rpDistanceMatrix(observation1,observation2) = sqrt(sum((rpX(observation1,:)-rpX(observation2,:)).^2));
                rpDistanceMatrix(observation2,observation1) = rpDistanceMatrix(observation1,observation2);
            end
        end
    %     distanceMatrixSize = size(distanceMatrix,1)*size(distanceMatrix,2);
    %     distanceSamples = ceil(distanceMatrixSize*rand(10000,1));
    %     reportPlot('scatter','Frame distance vs. approximated distance',...
    %                 distanceMatrix(distanceSamples),'True distance',...
    %                 rpDistanceMatrix(distanceSamples),'Approximated distance');
    %     corrOrigDistApproxDist = corr(distanceMatrix(:),rpDistanceMatrix(:));
    
        diffDistanceMatrix = rpDistanceMatrix ./ distanceMatrix;
        nonNaNdiffDistanceMatrix = diffDistanceMatrix(~isnan(diffDistanceMatrix));
        [muhat, sigmahat ] = normfit(nonNaNdiffDistanceMatrix);
        minVector(dim) = min(nonNaNdiffDistanceMatrix);
        maxVector(dim) = max(nonNaNdiffDistanceMatrix);
        upperStdVector(dim) = muhat + sigmahat;
        lowerStdVector(dim) = muhat-sigmahat;
    %     figure,boxplot(diffDistanceMatrix(:));
    end
end

%% Visulize results
reportPlot('plot','Distance distortion using random projections',...
        100:100:8000,'Projection space dimension',...
        [ 1-theoreticalBounds(1:80), minVector(1:80), lowerStdVector(1:80), upperStdVector(1:80), maxVector(1:80), 1+theoreticalBounds(1:80) ],'Distortion (1=no distortion)');
legend('Lower theoretical bound','Minimum','Lower \sigma','Upper \sigma','Maximum','Upper theoretical bound');