%% DESCRIPTION
% Scale video frames to a very low resolution (e.g 30x50). Each pixel is
% treated as a dimension. The distance between frames is calculated and
% searched for significant pattern.
% We then apply random projections and verify a correlation between the
% original distance and the low-dimension distance.

close all;
if(~exist('small_mov','var'))
    clear all;
    run('../copy_movie_to_smaller_array.m');
end
addpath('..');

lowWidth = 40;
lowHeight = 30;
dimension = lowWidth*lowHeight;
%nFrames=960;

%% Load low-res frames and vectorize
X = zeros(nFrames,dimension);
for frame=1:nFrames
    Ilowgray = imresize(small_mov(frame).cdata,[ lowHeight, lowWidth]);
    X(frame,:) = Ilowgray(:);
end

%% Calculate distance between frames
distanceMatrix = zeros(nFrames);
for frame1=1:nFrames
    for frame2=(frame1+1):nFrames
        distanceMatrix(frame1,frame2) = sqrt(sum((X(frame1,:)-X(frame2,:)).^2));
        distanceMatrix(frame2,frame1) = distanceMatrix(frame1,frame2);
    end
end
figure, imagesc(distanceMatrix);

%% Random projection
% Generate random matrix with normal distributed entries
subDimension = 10*ceil(log(dimension));
RP = mvnrnd(zeros(subDimension,dimension),eye(dimension));
rpX = X*RP';
rpDistanceMatrix = zeros(nFrames);
for frame1=1:nFrames
    for frame2=(frame1+1):nFrames
        rpDistanceMatrix(frame1,frame2) = sqrt(sum((rpX(frame1,:)-rpX(frame2,:)).^2));
        rpDistanceMatrix(frame2,frame1) = rpDistanceMatrix(frame1,frame2);
    end
end
figure, imagesc(rpDistanceMatrix);
distanceMatrixSize = size(distanceMatrix,1)*size(distanceMatrix,2);
distanceSamples = ceil(distanceMatrixSize*rand(10000,1));
reportPlot('scatter','Frame distance vs. approximated distance',...
            distanceMatrix(distanceSamples),'True distance',...
            rpDistanceMatrix(distanceSamples),'Approximated distance');
corrOrigDistApproxDist = corr(distanceMatrix(:),rpDistanceMatrix(:));

%% Detect scene shifts
% frameDifference = zeros(nFrames-1,1);
% for frame=2:nFrames
%     frameDifference(frame-1) = distanceMatrix(frame-1,:)*distanceMatrix(frame,:)';
% end
% figure, plot(2:nFrames,frameDifference); % Doesn't work!
reportPlot('plot','Frame difference',...
            2:nFrames,'#Frame pair',...
            diag(distanceMatrix,1),'Frame difference (|| x ||_2)');
        xlim([0, nFrames]);
return;

%% Detect scene category
%load('../cutdetection/bang-youre-dead_cuts.mat'); % Load cuts of the movie
cuts = [ 1, 16, 67, 137, 187, 244, 272, 347, 399, 416, 431 ]; % Cuts for Zelotypia 1-min
sceneCount = length(cuts)-1;
sceneCompareMatrixMin = zeros(sceneCount);
sceneCompareMatrixMean = zeros(sceneCount);
for scene1=1:sceneCount
    rowStart = cuts(scene1);
    rowStop = cuts(scene1+1)-1;
    for scene2=1:sceneCount
        colStart = cuts(scene2);
        colStop = cuts(scene2+1)-1;
        distanceSubmatrix = rpDistanceMatrix(rowStart:rowStop,colStart:colStop);
        sceneCompareMatrixMin(scene1,scene2) = min(distanceSubmatrix(:));
        sceneCompareMatrixMean(scene1,scene2) = mean(distanceSubmatrix(:));
    end
end
minSceneFrameDistanceTrueRelation = sceneCompareMatrixMin(sceneCategory);
minSceneFrameDistanceNoRelation = sceneCompareMatrixMin(~sceneCategory);
meanSceneFrameDistanceTrueRelation = sceneCompareMatrixMean(sceneCategory);
meanSceneFrameDistanceNoRelation = sceneCompareMatrixMean(~sceneCategory);

[ minTrueRelationHist, xTrueRelation ] = hist(minSceneFrameDistanceTrueRelation(:),20);
[ minNoRelationHist, xNoRelation ] = hist(minSceneFrameDistanceNoRelation(:),20);
bar(xNoRelation,minNoRelationHist);
hold on;
bar(xTrueRelation,minTrueRelationHist,'r');
hold off;


