function fig = reportPlot( plottype, plotTitle, X, plotXlabel, Y, plotYlabel, Z, plotZlabel )
%REPORTPLOT Make plots for reports. Figure handle is returned.
%   plottype:   plot, line, hist, bar, scatter, semilogx, semilogy, loglog,
%   trisurf
%   plotTitle:  The title of the plot
%   X:  X-data
%   plotXlabel: Description of the X-data
%   Y:  Y-data
%   plotYlabel: Description of Y-data

if(nargin < 6)
    error('Not enough input arguments');
end

run('plot_settings.m');
zDataPresent = (nargin > 7);

fig = figure();

if(strcmpi(plottype,'plot'))
    plot(X,Y,'O-','LineWidth',plotLineWidth+1);
elseif(strcmpi(plottype,'line'))
    plot(X,Y,'-','LineWidth',plotLineWidth+1);
elseif(strcmpi(plottype,'hist'))
    hist(Y,X);
elseif(strcmpi(plottype,'bar'))
    bar(X,Y);
elseif(strcmpi(plottype,'scatter'))
    if(zDataPresent)
        scatter(X,Y,20,Z);
    else
        scatter(X,Y);
    end
elseif(strcmpi(plottype,'semilogx'))
    semilogx(X,Y,'LineWidth',plotLineWidth+1);
elseif(strcmpi(plottype,'semilogy'))
    semilogy(X,Y,'LineWidth',plotLineWidth+1);
elseif(strcmpi(plottype,'loglog'))
    loglog(X,Y,'LineWidth',plotLineWidth+1);
elseif(strcmpi(plottype,'trisurf'))
    if(~zDataPresent)
        error('Missing Z-data')
    end
    tri = delaunay(X,Y);
    trisurf(tri,X,Y,Z);
end
if(~strcmpi(plottype,'hist'))
    xlim([min(X), max(X)]);
end
title(plotTitle,'FontSize',plotTitleFontSize);
xlabel(plotXlabel,'FontSize',plotLabelFontSize);
ylabel(plotYlabel,'FontSize',plotLabelFontSize);
if(zDataPresent)
    zlabel(plotZlabel,'FontSize',plotLabelFontSize);
end
%legend('Data','FontSize',plotLegendFontSize);
pos = get(gcf, 'Position');
set(gcf, 'Position', [pos(1) pos(2) plotWidth*100, plotHeight*100]);
set(gca, 'LineWidth', plotLineWidth);
set(gca, 'Box', 'off');


end

