%% DESCRIPTION
% Visualize the variance of linear regression for sampled data.

close all;
clear all;
addpath('../leverageScores');
run('../plot_settings.m');

% Set parameters
n = 300;
p = 1;
noiseLevel = 1.5;
distribution = 'normal';
df = 3;
sampleSize = 50;
repetitions = 30;

% Generate artificial data
[ X, Y, beta ] = generate_data(n,p,noiseLevel,distribution,df);
minX = min(X)-1;
maxX = max(X)+1;
H = X*((X'*X) \ X');
h = diag(H) / p;

%% Full regression
figure(1), dataHandle = plot(X,Y,'.'); xlabel('Data'), ylabel('Response'), xlim([-4, 4]), ylim([-10, 10]);
title('OLS linear regression');
hold on;
betaHat = (X'*X) \ (X'*Y);
minY = minX*betaHat;
maxY = maxX*betaHat;
fullHandle = plot([minX; maxX],[minY; maxY],'r','LineWidth',4);
legend([dataHandle, fullHandle],'Data','Full regression','Location','NorthWest');
hold off;
return;

%% Unfair sampling
figure(2), dataHandle = plot(X,Y,'.'); xlabel('Data'), ylabel('Response'), xlim([-4, 4]), ylim([-10, 10]);
title('Unfair sampling');
hold on;
for i=1:repetitions
    sample = (rand(n,1) < 0.5) & (abs(Y) > abs(X));
    Xsample = X(sample);
    betaHat = (Xsample'*Xsample) \ (Xsample'*Y(sample));
    minY = minX*betaHat;
    maxY = maxX*betaHat;
    sampleHandle = plot([minX; maxX],[minY; maxY],'k','LineWidth',1);
end
plot(Xsample,Y(sample),'kO','LineWidth',2);
% Full solution
betaHat = (X'*X) \ (X'*Y);
minY = minX*betaHat;
maxY = maxX*betaHat;
fullHandle = plot([minX; maxX],[minY; maxY],'r','LineWidth',4);
legend([dataHandle, sampleHandle, fullHandle],'Data','Sampled regression','Full regression','Location','NorthWest');
hold off;

%% Uniform sampling
figure(3), dataHandle = plot(X,Y,'.'); xlabel('Data'), ylabel('Response'), xlim([-4, 4]), ylim([-10, 10]);
title('Uniform sampling');
hold on;
for i=1:repetitions
    sample = rand(n,1) < sampleSize/n;
    Xsample = X(sample);
    betaHat = (Xsample'*Xsample) \ (Xsample'*Y(sample));
    minY = minX*betaHat;
    maxY = maxX*betaHat;
    sampleHandle = plot([minX; maxX],[minY; maxY],'k','LineWidth',1);
end
plot(Xsample,Y(sample),'kO','LineWidth',2);
% Full solution
betaHat = (X'*X) \ (X'*Y);
minY = minX*betaHat;
maxY = maxX*betaHat;
fullHandle = plot([minX; maxX],[minY; maxY],'r','LineWidth',4);
legend([dataHandle, sampleHandle, fullHandle],'Data','Sampled regression','Full regression','Location','NorthWest');
hold off;

%% Leverage sampling
figure(4), dataHandle = plot(X,Y,'.'); xlabel('Data'), ylabel('Response'), xlim([-4, 4]), ylim([-10, 10]);
title('Leverage sampling');
hold on;
for i=1:repetitions
    sample = nonuniform_sampling(h,sampleSize);
    Xsample = X(sample);
    betaHat = (Xsample'*Xsample) \ (Xsample'*Y(sample));
    minY = minX*betaHat;
    maxY = maxX*betaHat;
    sampleHandle = plot([minX; maxX],[minY; maxY],'k','LineWidth',1);
end
plot(Xsample,Y(sample),'kO','LineWidth',2);
% Full solution
betaHat = (X'*X) \ (X'*Y);
minY = minX*betaHat;
maxY = maxX*betaHat;
fullHandle = plot([minX; maxX],[minY; maxY],'r','LineWidth',4);
legend([dataHandle, sampleHandle, fullHandle],'Data','Sampled regression','Full regression','Location','NorthWest');
hold off;
