%% DESCRIPTION
% Visualize the variance of logistic regression for random sampling. Also
% illustrate the variance of \beta for nearly separable data.

close all;
clear all;
run('../plot_settings.m');

n = 300;
sampleSize = 100;
experiments = 50;

% Create artificial data
steepness = 10;
beta0 = [ -steepness/2; steepness ];
X = sort(rand(n,1));
p0 = exp(beta0(1)+X*beta0(2)) ./ (1+exp(beta0(1)+X*beta0(2)));
y = (rand(n,1) > p0) + 1;

% Visualize artificial data
figure(1), plot(X,p0,'-','LineWidth',6);
figure(2), plot(X,y-1,'or'), ylim([-0.2 1.2]);

%% UNCONDITIONAL FULL DATA SETS
% betaDistance = zeros(experiments,1);
% for i=1:experiments
%     X = sort(rand(n,1));
%     p0 = exp(beta0(1)+X*beta0(2)) ./ (1+exp(beta0(1)+X*beta0(2)));
% %     figure(1), hold on; plot(X,p0,'O'); hold off;
%     y = (rand(n,1) > p0) + 1;
% 
%     beta = mnrfit(X,y);
%     pHat = exp(beta(1)+X*beta(2)) ./ (1+exp(beta(1)+X*beta(2)));
%     figure(1); hold on; plot(X,pHat,'k'); hold off;
%     betaDistance(i) = sqrt(sum((beta-beta0).^2));
% end
% figure(3); hist(betaDistance,10);
% waitforbuttonpress;

%% SINGLE DATA SET SAMPLED
% figure(1), plot(X,p0,'-','LineWidth',6);
betaDistance = zeros(experiments,1);
for i=1:experiments
    sample = rand(n,1) < sampleSize/n;
    Xsample = X(sample);
    ysample = y(sample);
    
    beta = mnrfit(Xsample,ysample);
    pHat = exp(beta(1)+X*beta(2)) ./ (1+exp(beta(1)+X*beta(2)));
    figure(2); hold on; plot(X,pHat,'k'); hold off;
    betaDistance(i) = sqrt(sum((beta-beta0).^2));
end
figure(2), hold on; plot(X,p0,'-','LineWidth',6); hold off;
xlabel('Data'), ylabel('Probability / label');
figure(3); hist(betaDistance,10);