%% DESCRIPTION
% This script approximates an image with random samples which are
% interpolated in the reconstruction process.
% The sampling is done iteratively, sampling a fixed number of samples in
% each round, using the difference-image as weighting function for the next
% random draw.

close all;
if(~exist('small_mov','var'))
    clear all;
    run('../copy_movie_to_smaller_array.m');
end
addpath('../leverageScores');

%% Preprocessing
disp(strcat('Frame',num2str(i)));
try
    Igray = rgb2gray(small_mov(i).cdata);
catch
    Igray = small_mov(i).cdata(:,:,1);
end
% Igray = rgb2gray(imread('../../images/12_small.jpg'));

%% Settings
initialSampleSize = max(size(Igray));
maxIterations = min(size(Igray));
sampleSize = initialSampleSize;
sampleGrowthRate = 2;
pixelCount = size(Igray,2)*size(Igray,1);

[Xgrid, Ygrid] = meshgrid(1:size(Igray,2),1:size(Igray,1));
weights = ones(pixelCount,1) / pixelCount;
Isample = zeros(size(Igray));
maxDifference = 255;
totalDifference = 255;
totalSampleIndex = [ 1; size(Igray,1); pixelCount-size(Igray,1)+1; pixelCount ];
totalSampleSize = 4;
statistics = zeros(maxIterations,4); % Column 1: totalSampleSize
                                     % Column 2: totalDifference
                                     % Column 3: maxDifference
                                     % Column 4: elapsedTime

i = 1;


%% Start iterations
iteration = 1;
tic;
while(totalDifference > 0.1 && iteration < maxIterations)
    %% Sampling
    sampleIndex = nonuniform_sampling(weights,sampleSize);
    totalSampleIndex = [ totalSampleIndex ; sampleIndex ];
    totalSampleSize = totalSampleSize + sampleSize;
    sample = [ Ygrid(totalSampleIndex), Xgrid(totalSampleIndex) ];
    sampleValues = zeros(totalSampleSize,1);
    for j=1:totalSampleSize
        sampleValues(j) = double(Igray(sample(j,1),sample(j,2)));
        Isample(sample(j,1),sample(j,2)) = 1;
    end
    
    %% Reconstruction
    Ireconst = griddata(sample(:,2),sample(:,1),sampleValues,Xgrid,Ygrid);
    Ireconst(isnan(Ireconst)) = 0;
    figure(1), image(Ireconst*0.3), colormap gray, axis image off, title('Reconstructed image');
    
    differenceImage = abs(double(Igray) - Ireconst);
    maxDifference = max(differenceImage(:));
    totalDifference = sum(differenceImage(:)) / pixelCount;
    figure(2), image(differenceImage), colormap gray, axis image off, title(strcat('Difference image, totalDifference:',num2str(totalDifference)));
    
    statistics(iteration,:) = [ totalSampleSize*100/pixelCount, totalDifference, maxDifference, toc ];
    
    %% Prepare sampleSize and weights for next iteration
    iteration = iteration + 1;
    sampleSize = initialSampleSize*iteration;
    for j=1:pixelCount
        weights(j) = differenceImage(Ygrid(j),Xgrid(j));
    end
    weights = weights / sum(weights);
    figure(3), image(reshape(weights,size(Igray))*pixelCount*2), axis image off, title('Weights');
end

figure, image(Isample*60), colormap gray, axis image off, title(strcat('Samples ',num2str(totalSampleSize),'=',num2str(totalSampleSize*100/pixelCount),'%'));