%% DESCRIPTION
% Images are often broken down into blocks on which PCA is performed.
% This script tests the signal-to-noise-ratio for PCA for a given sample
% percentage and different block sizes. The script is used to determine the
% optimal block size.

close all;
if(~exist('small_mov','var'))
    clear all;
    run('../copy_movie_to_smaller_array.m');
end

sampleRate = 0.5;
blockSizeVector = [ 1:9, 10:5:50 ]; % This refers to the side length of a block
snrPCAVector2 = zeros(length(blockSizeVector),15);

% Perform analysis on several frames (15)
for frame=1:15
    Igray = small_mov(frame).cdata;
    % Test the specified block sizes
    for i=1:length(blockSizeVector)
        blockSize = blockSizeVector(i);
        pcaIreconst = pca_reconstruct(Igray,blockSize,sampleRate);
        Iwidth = size(Igray,2)-mod(size(Igray,2),10);
        Iheight = size(Igray,1)-mod(size(Igray,1),10);
        pcaReconstCrop = pcaIreconst(1:Iheight,1:Iwidth);
        diffImage = abs(double(Igray(1:Iheight,1:Iwidth)) - double(pcaReconstCrop));
        snrPCAVector2(i,frame) = nanmean(pcaReconstCrop(:)) / nanvar(diffImage(:));
    end
end
plot(blockSizeVector,mean(snrPCAVector2,2));
xlabel('Block size'), ylabel('SNR'),title(strcat('SNR for sample rate of ',num2str(sampleRate*100),'%'));
