%% DESCRIPTION %%
% This script takes images and samples random (possibly weighted) pixels
% from the original image and reconstructs the image from these sample
% points by interpolation
% The interpolation is done by building a surface from the sample-points,
% where the surface is a linear interpolation of the triangles making up
% the underlying mesh.


% close all;
if(~exist('small_mov','var'))
    clear all;
    run('../copy_movie_to_smaller_array.m');
end
addpath('../leverageScores');
run('../vlfeat-0.9.18/toolbox/vl_setup.m');

sampleRate = 0.1;
sampleSize = round(sampleRate*smallWidth*smallHeight);

% nFrames = 200;
snrVector = zeros(nFrames,1);
snrRegularVector = zeros(nFrames,1);
snrPCAVector = zeros(nFrames,1);

%% Create and position figures
figure(1);
set(gcf,'Units','normalized');
set(gcf,'OuterPosition',[ 0, 0.5, 0.5, 1]);
figure(2);
set(gcf,'Units','normalized');
set(gcf,'OuterPosition',[ 0.5, 0.66, 0.5, 0.33]);
figure(3);
set(gcf,'Units','normalized');
set(gcf,'OuterPosition',[ 0.5, 0.33, 0.5, 0.33]);
figure(4);
set(gcf,'Units','normalized');
set(gcf,'OuterPosition',[ 0.5, 0, 0.5, 0.33]);

[ Xgrid, Ygrid ] = meshgrid(1:smallWidth,1:smallHeight);
samplePoints = zeros(sampleSize+4,1);

linearInterpolationMovie = zeros(smallHeight,smallWidth,1,nFrames);
roimask = false(smallHeight,smallWidth);
for i=1:nFrames
   %% Preprocessing
   disp(strcat('Frame',num2str(i)));
   Igray = small_mov(i).cdata;
   %B = edge(Igray,'canny');
   [B,~] = imgradient(Igray);
   averageSize = 10;
   blurFilter = fspecial('average',[averageSize averageSize]);
   Bblur = imfilter(single(B),blurFilter);
   nearEdge = Bblur - B./averageSize^2; %((Bblur - B) > 0);
   % Experimental: Set weight outside the ROI to 10%. ROI manually chosen.
   if(i < nFrames)
       [ mask, clusterSizeRatio, meanDistance ] = roi_selection(small_mov(i).cdata,small_mov(i+1).cdata);
       if(clusterSizeRatio <= 2)
           roimask = mask;
       elseif(clusterSizeRatio < 20)
           roimask = roimask | mask;
       end
   end
   nearEdge(~roimask) = nearEdge(~roimask)*0.1;
%    nearEdge(:,1:100) = nearEdge(:,1:100)*0.1;
%    nearEdge(:,350:end) = nearEdge(:,350:end)*0.1;

   
   %% Sampling distribution
   % Uniform random sampling
%    sample = ceil(rand(sampleSize,2)*[ smallHeight, 0 ;  0, smallWidth]);
   
   % Edge-weighting
   weights = double(ones(size(Igray)) + 200*(nearEdge));
   weights = weights / sum(weights(:));
   sampleIndex = nonuniform_sampling(weights(:),sampleSize);
   sampleIndex = [ sampleIndex ; 1 ; smallHeight ; smallHeight*(smallWidth-1)+1 ; smallHeight*smallWidth ];
   sample = [ Ygrid(sampleIndex), Xgrid(sampleIndex) ];
   
   % Number of points sample more than once:
%    duplicates = zeros(sampleSize-1,1);
%    sampleIndex = sort(sampleIndex);
%    for j=1:sampleSize-1
%       duplicates(j) = sampleIndex(j) - sampleIndex(j+1);
%    end
%    duplicates = (duplicates == 0);
%    disp(strcat('Duplicates:',num2str(sum(duplicates))));
   
   
   %% Sampling
   Isample = zeros(size(Igray));
   for j=1:sampleSize+4
       samplePoints(j) = double(Igray(sample(j,1),sample(j,2)));
       Isample(sample(j,1),sample(j,2)) = 1;
   end
   
   %% Reconstruction
   Ireconst = griddata(sample(:,2),sample(:,1),samplePoints,Xgrid,Ygrid);
   pcaIreconst = pca_reconstruct(Igray,10,sampleRate);
   Idownsample = imresize(imresize(Igray,sqrt(sampleRate)),[smallHeight, smallWidth ]);
   
   %% Analysis
   % SNR of full frame
   diffImage = abs(double(Igray) - Ireconst);
   snrVector(i) = nanmean(Ireconst(:)) / nanvar(diffImage(:));
   
   % SNR of regular downsampled frame
   diffImage = abs(double(Igray) - double(Idownsample));
   snrRegularVector(i) = nanmean(Idownsample(:)) / nanvar(diffImage(:));
   % SNR of PCA reconstruction:
   Iwidth = size(Igray,2)-mod(size(Igray,2),10);
   Iheight = size(Igray,1)-mod(size(Igray,1),10);
   pcaReconstCrop = pcaIreconst(1:Iheight,1:Iwidth);
   diffImage = abs(double(Igray(1:Iheight,1:Iwidth)) - double(pcaReconstCrop));
   snrPCAVector(i) = nanmean(pcaReconstCrop(:)) / nanvar(diffImage(:));
   
    % SNR of ROI
%    ROIreconstr = Ireconst(:,101:349);
%    diffImage = abs(double(Igray(:,101:349)) - Ireconst(:,101:349));
%    snrVector(i) = nanmean(ROIreconstr(:)) / nanvar(diffImage(:));
   
   %% Make a movie
   linearInterpolationMovie(:,:,1,i) = (Ireconst);
   
   %% Visualization
   figure(1);
   imagesc(cat(1,Idownsample,Igray,Ireconst)), colormap gray;
   title('Reconstructed image');
   figure(2);
   image(diffImage), axis image;
   title('Difference');
   figure(3);
   image(Isample*60), axis image, colormap gray;
   title('Sample points');
   figure(4);
   image(weights*60 / max(weights(:))), axis image, colormap gray;
   %break;
%    waitforbuttonpress;
    
end

% for i=1:40
%     image(linearInterpolationMovie(:,:,1,i)*0.25), axis image, colormap gray;
%     waitforbuttonpress;
% end