%% DESCRIPTION %%
% This script takes images and samples random (possibly weighted) pixels
% from the original image and reconstructs the image from these sample
% points by interpolation
% Difference to randomSamplingInterpolation.m: This script uses linear
% regression with the k-nearest points for interpolation of the pixels.

close all;
if(~exist('small_mov','var'))
    clear all;
    run('../copy_movie_to_smaller_array.m');
end
addpath('../leverageScores');

sampleRate = 0.02;
sampleSize = round(sampleRate*smallWidth*smallHeight);
k = 20; % Number of interpolation points for linear regression

nFrames = 40;

%% Create and position figures
figure(1);
set(gcf,'Units','normalized');
set(gcf,'OuterPosition',[ 0, 0.5, 0.5, 1]);
figure(2);
set(gcf,'Units','normalized');
set(gcf,'OuterPosition',[ 0.5, 0.66, 0.5, 0.33]);
figure(3);
set(gcf,'Units','normalized');
set(gcf,'OuterPosition',[ 0.5, 0.33, 0.5, 0.33]);
figure(4);
set(gcf,'Units','normalized');
set(gcf,'OuterPosition',[ 0.5, 0, 0.5, 0.33]);

[ Xgrid, Ygrid ] = meshgrid(1:smallWidth,1:smallHeight);
sampleValue = zeros(sampleSize,1);

linearInterpolationMovie = zeros(smallHeight,smallWidth,1,nFrames);
for i=1:nFrames
   %% Preprocessing
   disp(strcat('Frame',num2str(i)));
   try
    Igray = rgb2gray(small_mov(i).cdata);
   catch
    Igray = small_mov(i).cdata(:,:,1);
   end
   %B = edge(Igray,'canny');
   [B,~] = imgradient(Igray);
   averageSize = 10;
   blurFilter = fspecial('average',[averageSize averageSize]);
   Bblur = imfilter(single(B),blurFilter);
   nearEdge = Bblur - B./averageSize^2; %((Bblur - B) > 0);
   % Experimental: Set weight outside the ROI to 10%. ROI manually chosen.
   if(i < nFrames)
       [ mask, clusterSizeRatio, meanDistance ] = roi_selection(small_mov(i).cdata,small_mov(i+1).cdata);
       if(clusterSizeRatio <= 2)
           roimask = mask;
       elseif(clusterSizeRatio < 20)
           roimask = roimask | mask;
       end
   end
   nearEdge(~roimask) = nearEdge(~roimask)*0.1;
%    nearEdge(:,1:100) = nearEdge(:,1:100)*0.1;
%    nearEdge(:,350:end) = nearEdge(:,350:end)*0.1;

   
   %% Sampling distribution
   % Uniform random sampling
   sample = ceil(rand(sampleSize,2)*[ smallHeight, 0 ;  0, smallWidth]);
   
   % Edge-weighting
%    weights = double(ones(size(Igray)) + 200*(nearEdge));
%    weights = weights / sum(weights(:));
%    sampleIndex = nonuniform_sampling(weights(:),sampleSize);
%    sample = [ Ygrid(sampleIndex), Xgrid(sampleIndex) ];
   
   % Number of points sample more than once:
%    duplicates = zeros(sampleSize-1,1);
%    sampleIndex = sort(sampleIndex);
%    for j=1:sampleSize-1
%       duplicates(j) = sampleIndex(j) - sampleIndex(j+1);
%    end
%    duplicates = (duplicates == 0);
%    disp(strcat('Duplicates:',num2str(sum(duplicates))));
   
   
   %% Sampling
   Isample = zeros(size(Igray));
   for j=1:sampleSize
       sampleValue(j) = double(Igray(sample(j,1),sample(j,2)));
       Isample(sample(j,1),sample(j,2)) = 1;
   end
   
   %% Reconstruction
   Ireconst = zeros(size(Igray));
   for j=1:length(Xgrid(:))
       % Calculate distance of sample points to current pixel
       distance = [ (1:sampleSize)', sqrt((sample(:,2)-Xgrid(j)).^2 + (sample(:,1)-Ygrid(j)).^2) ];
       % Sort the rows of the distance matrix, which has the index of the
       % sample in the first row.
       sortedDistance = sortrows(distance,2);
       % Build data matrix
       X = [ ones(k,1), sample(sortedDistance(1:k,1),:)];
       meanY = mean(X(:,2));
       meanX = mean(X(:,3));
       X(:,2) = X(:,2)-meanY;
       X(:,3) = X(:,3)-meanX;
       % Visualize distances
       if(j == 5000)
           scatter(Xgrid(j),Ygrid(j),50,'red'); hold on;
           scatter(sample(sortedDistance(1:k,1),1),sample(sortedDistance(1:k,1),2),20,distance(sortedDistance(1:k,1),2)); hold off;
           axis([ 1 smallWidth 1 smallHeight]);
           %waitforbuttonpress;
       end
       % Build the response vector
       nearestSamplesValue = sampleValue(sortedDistance(1:k,1));
       % Perform linear regression
       beta = (X'*X) \ (X'*nearestSamplesValue);
       % Estimate pixel value
       yhat = [ 1, Ygrid(j)-meanY, Xgrid(j)-meanX ]*beta;
       if( yhat > 255)
           scatter(Xgrid(j),Ygrid(j),50,'red'); hold on;
           scatter(sample(sortedDistance(1:k,1),1),sample(sortedDistance(1:k,1),2),20,distance(sortedDistance(1:k,1),2)); hold off;
           axis([ 1 smallWidth 1 smallHeight]);
           disp('!');
       end
       Ireconst(j) = min(max(floor(yhat),0),255);
   end
   Idownsample = imresize(imresize(Igray,sqrt(sampleRate)),[smallHeight, smallWidth ]);
   
   %% Make a movie
   linearInterpolationMovie(:,:,1,i) = (Ireconst);
   
   %% Visualization
   figure(1);
   imagesc(cat(1,Idownsample,Igray,Ireconst)), colormap gray;
   title('Reconstructed image');
   figure(2);
   image(abs(double(Igray) - Ireconst)), axis image;
   title('Difference');
   figure(3);
   image(Isample*60), axis image, colormap gray;
   title('Sample points');
   figure(4);
   image(weights*60 / max(weights(:))), axis image, colormap gray;
   break;
   %waitforbuttonpress;
    
end

% for i=1:40
%     image(linearInterpolationMovie(:,:,1,i)*0.25), axis image, colormap gray;
%     waitforbuttonpress;
% end