function [ dataset ] = build_dataset( mov )
%BUILD_DATASET Build dataset for the input movie
%   For each frame in the supplied movie several features are computed
%   Input:  - mov, a movie struct
%   Output: - data, a dataset with the following properties
% Dataset:
% - Observations:   #frames
% - Features:   * 1: Scenenr
%               * 2: Intensity difference
%               * 3: Normalized ROI area
%               * 4: Camera motion characteristics

    featureCount = 4;
    nFrames = length(mov);
    [ vHeight, vWidth ] = size(mov(1).cdata);
    
    dataset = zeros(nFrames,featureCount);
    
    cuts = detect_cuts(mov);
    dataset(:,1) = scenenr(cuts,nFrames);
    for i=1:nFrames
        if(i > 1)
            dataset(i,2) = intensity_difference(mov,i);
        end
        if(i < nFrames)
            mask = roi_select(mov,i);
            dataset(i,3) = sum(mask(:)) / (vHeight*vWidth);
        end
        if(mod(i,20) == 0)
            % Output progress
            disp(strcat('Frame ',num2str(i),' processed'));
        end
    end
    dataset(:,4) = camera_motion_characteristics(mov,cuts);

end

