function [ mov ] = load_video( path, targetFrameRate, maxFrameCount )
%LOAD_VIDEO Load a video from a file
%   Input:  - path
%           - targetFrameRate (> 0 or NaN for original frame rate)
%           - maxFrameCount
%   Output: A gray scale video structure

% Define path to video file
videoObj = VideoReader(path);

% Downsample framerate to a chosen level
if(nargin == 1 || isnan(targetFrameRate))
    frameRateRescale = 1;
else
    frameRateRescale = videoObj.FrameRate / targetFrameRate;
end

% Get number and size of frames (with new framerate)
nFrames = floor(videoObj.NumberOfFrames / frameRateRescale);
nFrames = min(nFrames,maxFrameCount);
vidHeight = videoObj.Height;
vidWidth = videoObj.Width;

% Preallocate movie structure.
mov(1:nFrames) = struct('cdata', zeros(vidHeight, vidWidth, 'uint8'),...
                        'colormap', []);

% Read one frame at a time.
for k = 1 : nFrames
    mov(k).cdata = rgb2gray(read(videoObj, floor(k*frameRateRescale)));
    if(mod(k,20) == 0)
        % Output progress
        disp(strcat('Frame ',num2str(k),' loaded'));
    end
end
end

